from .version import VERSION
__version__ = VERSION

from .client import Client
from .pages import Pages
from .patch import Patch
from .resource import Resource
from .response import Response
from . import util



from .resource import Resource
from collections import Iterator
import copy
try:
    # python 2
    from urllib import quote
except ImportError:
    # python 3
    from urllib.parse import quote


class Pages(Iterator):

    def __init__(self, opts, url, path, params):
        if isinstance(path, list):
            pages_url = '/'.join([url] + [quote(elem) for elem in path])
        else:
            pages_url = '/'.join([url, quote(path)])
        self.resource = Resource(pages_url, **opts)
        self.params = params
        self._root_resource = Resource(url[:url.find('/v0')], **opts)
        self.response = None

    def _handle_page(self, querydict={}, val='next', **headers):
        """
        Executes the request getting the next (or previous) page,
        incrementing (or decrementing) the current page.
        """
        params = copy.copy(self.params)
        params.update(querydict)
        # update uri based on next page
        if self.response:
            self.response.raise_for_status()
            _next = self.response.links.get(val, {}).get('url')
            if _next:
                response = self._root_resource._make_request(
                    'GET', _next, params, **headers)
                self._handle_res(None, response)
                return response
            else:
                raise StopIteration
        else:
            response = self.resource._make_request(
                'GET', '', params, **headers)
            self._handle_res(None, response)
            return response

    def _handle_res(self, session, response):
        """
        Stores the response, which we use for determining
        next and prev pages.
        """
        self.response = response

    def reset(self):
        """
        Clear the page's current place.

            page_1 = page.next().result()
            page_2 = page.next().result()
            page.reset()
            page_x = page.next().result()
            assert page_x.url == page_1.url
        """
        self.response = None

    def next(self, querydict={}, **headers):
        """
        Gets the next page of results.
        Raises `StopIteration` when there are no more results.
        """
        return self._handle_page(querydict, **headers)

    def __next__(self):
        return self.next()

    def prev(self, querydict={}, **headers):
        """
        Gets the previous page of results.
        Raises `StopIteration` when there are no more results.

        Note: Only collection searches provide a `prev` value.
        For all others, `prev` will always return `StopIteration`.
        """
        return self._handle_page(querydict, 'prev', **headers)

    def all(self):
        results = []
        for response in self:
          response.raise_for_status()
          results.extend(response['results'])
        return results



#!/usr/bin/env python

import re
from setuptools import setup

VERSIONFILE="porc/version.py"
verstrline = open(VERSIONFILE, "rt").read()
VSRE = r"^VERSION = ['\"]([^'\"]*)['\"]"
mo = re.search(VSRE, verstrline, re.M)
if mo:
    VERSION = mo.group(1)
else:
    raise RuntimeError("Unable to find version string in %s." % (VERSIONFILE,))

setup(name='porc',
      version=VERSION,
      description='Asynchronous Orchestrate.io Interface',
      author='Max Thayer',
      author_email='max@orchestrate.io',
      url='https://github.com/orchestrate-io/porc',
      packages=['porc'],
      license='ASLv2',
      install_requires=[
          'requests-futures==0.9.4',
          'vcrpy==1.1.0',
          'lucene-querybuilder==0.2'
      ],
      test_suite="tests",
      classifiers=[
          'Intended Audience :: Developers',
          'Natural Language :: English',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4'
      ],
      )



'''
Paths can be separated by forward slashes: `"/name/first"`, or with dot
notation: `"name.first"`. If a path starts with a `'/'`, Orchestrate will be
interpret it as a [JSON Pointer](http://tools.ietf.org/html/rfc6901), otherwise
Orchestrate interprets the path as dot notation.
'''
class Patch:
    def __init__(self):
        self.operations = []

    '''
    Depending on the specified path, creates a field with that value, replaces
    an existing field with the specified value, or adds the value to an array.
    '''
    def add(self, path, value):
        return self.__operation('add', path, value)

    '''
    Removes the field at a specified path
    '''
    def remove(self, path):
        return self.__operation('remove', path)

    '''
    Replaces an existing value with the given value at the specified path.
    '''
    def replace(self, path, value):
        return self.__operation('replace', path, value)

    '''
    Moves a value from one path to another, removing the original path.
    '''
    def move(self, old_path, new_path):
        return self.__operation('move', new_path, from_path=old_path)

    '''
    Copies the value at one path to another.
    '''
    def copy(self, original, copy):
        return self.__operation('copy', copy, from_path=original)

    '''
    Tests equality of the value at a particular path to a specified value, the
    entire request fails if the test fails.
    '''
    def test(self, path, value):
        return self.__operation('test', path, value)

    '''
    Increments the numeric value at a specified field by the given numeric
    value, decrements if given numeric value is negative. Default is `1`
    '''
    def increment(self, path, value=1):
        return self.__operation('inc', path, value)

    '''
    Decrements the numeric value at a specified field by given a numeric value.
    This method is sugar. It wraps the `increment` method and multiplies the
    value by -1. Passing in a negative value will increment the field. The
    default is `-1`
    '''
    def decrement(self, path, value=1):
        self.increment(path, value*-1)
        return self

    def __operation(self, op, path, value=None, from_path=None):
        op = {
            'op': op,
            'path': path
        }
        if value != None:
            op['value'] = value
        if from_path:
            op['from'] = from_path

        self.operations.append(op)
        return self
